"""Provides a data model from a bank export in PDF format."""
from datetime import date, datetime, time
from re import Match
from typing import Any

from pydantic import BaseModel, computed_field, field_validator


class Operation(BaseModel):
    """Represents a single bank operation parsed from a PDF statement.

    Attributes:
        date (date): The date of the operation in format 'DD.MM.YYYY'.
        time (time): The time of the operation.
        category (str): The category or type of the operation.
        sum (float): The amount of money involved in the operation.
        remains (float): The remaining balance after the operation.
        is_income (bool): Whether the operation is an income (True) or expense (False).

    """

    date: date
    time: time
    category: str
    sum: float
    remains: float
    is_income: bool = False

    @field_validator("date", mode="before")
    def parse_date(cls, v: Any) -> Any:  # noqa N805
        """Parse a string into a date object using the format 'DD.MM.YYYY'.

        Use this validator to convert a string representation of a date into a Python
        date object before model initialization.

        Args:
            v (Any): The input value to be validated and converted to a date.

        Returns:
            date: A parsed date object if input is a valid string
            in 'DD.MM.YYYY' format.

        Raises:
            ValueError: If the string is not in the correct date format.

        """
        if isinstance(v, str):
            return datetime.strptime(v, "%d.%m.%Y").date() #noqa DTZ007
        return v

    @computed_field
    @property
    def weekday(self) -> int:
        """Return the ISO weekday number for the operation's date.

        Use this property to determine the day of the week according
        to ISO standard (Monday = 1, Sunday = 7).

        Returns:
            int: An integer between 1 (Monday) and 7 (Sunday).

        """
        return self.date.isoweekday()

    @classmethod
    def from_match(cls, match: Match, *, is_income: bool = False) -> "Operation":
        """Create an Operation instance from a regex match object.

        Use this class method to construct an Operation object from a regular
        expression match with named groups.

        Args:
            match (Match): A regex match object with named groups
            'date', 'time', 'category', 'sum', 'remains'.
            is_income (bool, optional): Indicates whether the operation is income.
            Defaults to False.

        Returns:
            Operation: A new Operation instance populated with values from the match.

        """
        date_ = match.group("date")
        return cls(
            date=date_,
            time=match.group("time"),
            category=match.group("category"),
            sum=match.group("sum"),
            remains=match.group("remains"),
            is_income=is_income,
        )
