"""Parse bank PDF statements for bank operations."""
import re
from re import Match

from .models import Operation


class SberParser:
    """Main class for parsing lines from a Sberbank PDF statement."""

    date_prefix = r"\d{2}\.\d{2}\.\d{4}"
    income = r"\W\+\S+\W"
    main_line = (r"(?P<date>\d{2}\.\d{2}\.\d{4})\W(?P<time>\d{2}\:\d{2})\W(\d+)\W"
                 r"(?P<category>\D+)\W(?P<sum>\S+)\W(?P<remains>\S+)")

    @staticmethod
    def _replace_nbsp(str_: str) -> str:
        """Replace non-breaking space ' ' with regular space.""" # noqa: RUF002
        return str_.replace(" ", "")

    @staticmethod
    def _replace_comma(str_: str) -> str:
        """Replace comma ',' with dot '.' for float parsing."""
        return str_.replace(",", ".")

    @classmethod
    def _is_operation(cls, str_: str) -> bool:
        """Check if the string starts with a valid date format."""
        return bool( re.match(pattern=cls.date_prefix, string=str_))

    @classmethod
    def _parse_main_line(cls, str_: str) -> Operation | None:
        """Parse a line into an Operation object if it matches the pattern.

        Return None if parsing fails.
        """
        operation: Operation | None = None

        str_ = cls._replace_nbsp(str_=str_)
        str_ = cls._replace_comma(str_=str_)

        if cls._is_operation(str_=str_):
            matched: Match | None = re.match(pattern=cls.main_line, string=str_)
            if matched:
                is_income = (
                    bool(re.search(pattern=cls.income, string=str_))
                )
                operation = Operation.from_match(match=matched, is_income=is_income)

        return operation

    @classmethod
    def parse_lines(cls, str_: str | list[str]) -> list:
        """Parse a list of strings or a single string into a list of Operation objects.

        Skip invalid lines.
        """
        operations: list[Operation] = []

        for line in str_:
            if re.match(pattern=cls.date_prefix, string=line):
                stat = cls._parse_main_line(str_=line)
                if stat:
                    operations.append(stat)

        return operations
